function M= selectFeatures_SHS(X,Y,gamma,miu, labelKernelType, matrixHalvingProcedure)
% INPUTS:
% X is the matrix of gene expression data. The size of X is n by p, where n
% represents the number of samples and p denotes the number of
% features. Input y is the response variable with dimension n by d. For
% classification problems, y is a 1-dim vertical vector and entries of y
% are integer values between 1 and nmax, where nmax is the number of classes.
% gamma and miu are parameters of the algorithm as determined in the paper.
% (Note that they are NOT gamma_bar and miu_bar).
% labelKernelType is the type of label kernels. It can take four values:
%
% 1- 'mod_delta' (most popular): The data dependent kernel mentioned in the
% paper. We use eigen value decompostion to halv the matrix in this method.
% 2- 'indicator': the zero-one block kernel. Not provided with this implementation.
% 3- 'linear': No change to Y.
% 4- 'rbf': Relevant only for continuous y labels. Uses median of distances for normalization.
% 
% matrixHalvingProcedure: Needs to be provided only when rbf kernel is used. It determines 
% how to deivide the label kernels into two halves. Each half is called
% delta in the paper. It can take two values: 'chol' (based on Cholesky
% decomposition) and 'eig' (based on eigen value decomposition). We have
% notes the 'eig' approach results in more resonable outcomes.
%
% OUTPUT:
% The output M represents the indices of selected features.

n = size(X,1);

switch labelKernelType
    case 'mod_delta'
        delta = mod_delta(X,Y);
    case 'indicator'
        delta= indicator(Y);
    case 'linear'
        delta= Y;
    case 'rbf'
        distanceVector = pdist(Y);
        distanceMatrix = squareform(distanceVector);        
        yKernel = exp(-distanceMatrix/median(distanceVector));
        delta = (matrix_half(yKernel, matrixHalvingProcedure))'; % Transpose is needed because of the way matrixHalf is defined/implemented.
end

H = eye(n)-1/n*ones(n,n);
A = X' * H * delta; % delta here is transpose of the delta intrduced in the paper. Also, X is transpose of X in the paper.

[M,sigma,u,v]=sparseSVD(A,gamma,miu);

M= find(M);


